% function ISIR_PR_demo
%--------------------------------------------------------------------------
%   Multiphase segmentation coupled with image restoration achievements
%
%   Usage: ISIR_PR_demo;
% 
%   Inputs: 
%       - none;
%
%   Outputs: 
%       - none;
% 
%   The original algorithm was proposed in the following paper, if you use 
%   this file or package for your work, please refer to the following
%   paper:
% 
%   [1] X. Cai. Variational Image Segmentation Model Coupled with Image 
%        Restoration Achievements. Pattern Recognition, 48:2029--2042, 2015
%  
%   parameters:
%       - beta: first regularization parameter
%       - mu: second regularization parameter
%       - c: phase number
% 
% 
%   All Rights Reserved  
%   Code by: Xiaohao Cai
%   Last updated: 20/07/2015 
%--------------------------------------------------------------------------

close all; clc; clear all;

%% load data
cd data
load crown_noise_gaussian_lost; 
% mask is the pixels with information loss
index_know=1-mask; F=Img; 

cd ..
%% set parameters
beta=100; mu=20; c=3;
%------others fixed parameters
lambda = 1/mu;
delta=1e-4;
eps=1e-4;
iter=100;
acsFlag= false;
gamma=2;

%% obtain initial codebook by k-means
tic
% k-means method to separate the given image into k parts
[xLen,yLen,zLen]=size(F);
v=zeros(xLen*yLen,3);
for i=1:3
    temp=F(:,:,i);
    v(:,i)=temp(:);
end
idx=kmeans(v,c); 

% compute the mean of each part
for i=1:c
    for j=1:3
        temp=double(i==idx).*v(:,j);
        vfcm(i,j)=sum(temp(:))/(sum(temp(:)>0));
    end
end
for i=1:c
    temp=double((i==idx));
    Ufcm(:,:,i)=reshape(temp,[xLen,yLen]);
end

%% segmentation
% compute U - labelling function in [0,1]
[U,W,v] = ...
    tvcm(F,Ufcm,vfcm,beta,lambda,gamma,delta,eps,iter,acsFlag,index_know);

% assign labels to obtain segmentation
[U_tv_lab,res_lab] = assignlabel(U,F,c);

toc

%% showing
figure,imshow(F,[],'border','tight');
figure,imshow(U_tv_lab,[],'border','tight');



