function [U,W,v,functional,abbruch,v_array,constraint_vio,t,outer] = ...
    tvcm( F,W,v,beta,lambda,gamma,delta,eps,iter,acsFlag,index_know)

% Solve argmin_{u,r} sum_jk u_k(j)||r_k-F(j)||_2^2 + lambda || |u| ||_1
%
% careful: in the paper we have a regularization parameter mu multiplied
% with the data term rather then with the regularizer.

% F:        input image
% c:        number of clusters
% W:        initialization label vector, size(m,n,c)
% v:        initialization codebook, size(c,1),
% gamma:    algorithm parameter
% iter:     number of iterations
% lambda:   regularizing parameter
% delta:    minimal descent (use 1e-4 and forget)
% eps:      oversegmentation check (use 1e-4 and forget)
% acsFlag:  0: Update codebook after a decrease of delta
%           1: ADMM is iterated until convergence before codebook
%           update resembling the alternate convex search algorithm
%
%

%nu = 2*mu*gamma;
lamgam = lambda*gamma;
[m,n,zLen] = size(F);
[c,c0] = size(v);

mn = m*n;

U = W;

% Forward difference matrices
[DDm,DDn] = first_order_differences(m,n);

% Cosine matrices for the inversion of the linear system
Cm = dctmtx(m);
Cn = dctmtx(n);
DD_Cm = Cm*(DDm'*DDm)*Cm';
DD_Cn = Cn*(DDn'*DDn)*Cn';
dm = diag(DD_Cm);
dn = diag(DD_Cn);
E = 1 ./ (1 + (kron(dm,ones(1,n)) + kron(ones(m,1),dn')));

% Initial values of the iterates
S = zeros(m,n,c);
Bd = zeros(2*m,n,c);
Bw = zeros(m,n,c);
D  = zeros(2*m,n,c);
% D*u
Nablau = zeros(2*m,n,c);

% just some values for analysis of the algorithm
data_t = zeros(iter,1);
tv_t = zeros(iter,1);
functional= zeros(iter,1);
outer=zeros(iter,1);
abbruch= zeros(iter,1);
sega_array= zeros(iter,1);
constraint_vio = zeros(iter,1);
v_array= zeros(iter,c);
t=0;
i_outer = 1;

% flag that indicates if the codebook should be updated
cbuFlag = true;
% functional to compare with to decide whether the codebook should
% be updated
functional_comp = realmax;

g=F;


for k = 1:iter
    U_old = U;
    
    %%%%%%%%%%%%%% updata g %%%%%%%%%%%%%%
    for j=1:3
        temp=zeros(m,n);
        for i=1:c
            temp=temp+v(i,j)*W(:,:,i);
        end
        rhs = beta*F(:,:,j)+temp;

        g(:,:,j)=index_know.*rhs./(beta+1);
    end
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    
    if cbuFlag
        outer(i_outer) = k;
        i_outer = i_outer+1;
        U_old_outer = U;
        cbuFlag = false;
        for i = 1 : c
            deno = sum(sum(index_know.*W(:,:,i)));
            % update codebook if denominator is not zero
            if deno > eps
                for j=1:3
                    v(i,j) = sum(sum(index_know.*W(:,:,i).*g(:,:,j)))/deno;
                end
            end
        end
        % compute S 
        for i = 1:c
            S(:,:,i) = index_know.*((g(:,:,1) - v(i,1)).^2+(g(:,:,2) - v(i,2)).^2+(g(:,:,3) - v(i,3)).^2);
        end        
    end
    
    
    for i = 1:c
        
        % right hand side of linear system
        RHS = DDm'*(D(1:m,:,i) -  Bd(1:m,:,i)) +...
            (D(m+1:2*m,:,i) - Bd(m+1:2*m,:,i))*DDn+ W(:,:,i) - Bw(:,:,i) - ...
            gamma*S(:,:,i);
        % solve linear system
        U(:,:,i) = Cm' * (E .* (Cm * RHS * Cn')) * Cn;
        
        % computing d by coupled shrinkage
        Nablau(1:m,:,i) = DDm* U(:,:,i);
        Nablau(m+1:2*m,:,i) = U(:,:,i)*DDn';
        Hv1 = Nablau(1:m,:,i) + Bd(1:m,:,i);
        Hv2 = Nablau(m+1:2*m,:,i) + Bd(m+1:2*m,:,i);
        T = sqrt(Hv1.^2 + Hv2.^2);
        
        D(1:m,:,i) = coupled_shrinkage(Hv1,T,lamgam);
        D(m+1:2*m,:,i) = coupled_shrinkage(Hv2,T,lamgam);
        
    end
    
    % Computing W by projection onto simplex
    p = reshape(Bw+U,mn,c);
    W = reshape(projection(p),m,n,c);
    %sum(w')
    
    % updating B
    for i = 1:c
        Bd(:,:,i) = Bd(:,:,i) + Nablau(:,:,i) - D(:,:,i);
        Bw(:,:,i) = Bw(:,:,i) + U(:,:,i) - W(:,:,i);
    end
    
    
    tStart = tic;
    data_term = sum(S(:).*W(:));    
    TV_term = 0;
    for i = 1:c
        TV_term = TV_term + ...
            sum(sum( sqrt( (DDm*W(:,:,i)).^2 + (W(:,:,i)*DDn').^2 ) ));
    end  
    tv_t(k) = lambda * TV_term;
    data_t(k) = data_term;
    t = t + toc(tStart);
    
    functional_current = data_term + lambda * TV_term;
    
    % abbruch means termination in german
    % defines a termination criterion that is printed in every iteration
    abbruch(k) = norm(U(:)-U_old(:))/norm(U(:));
    
    % check if the codebook should be recomputed
    if acsFlag == 0 && functional_current+delta <= functional_comp
        functional_comp = functional_current;
        cbuFlag = true;
    end
    functional(k) = functional_current;
    

%     v_array(k,:) = (v);
    
    % compute violation of the constraint Ax=y
    for i = 1:c
        constraint_vio(k) = constraint_vio(k) + ...
            norm([DDm*W(:,:,i);(W(:,:,i)*DDn')]-D(:,:,i), 'fro');
    end
    
    % print some algorithm data
    if(mod(k,round(iter/20))==0)
        fprintf('it: %g\ttermi: %g\tfunc: %g\tCVio: %g\n'...
            ,k,abbruch(k), functional(k), constraint_vio(k));
    end
	   
    
    if(abs(abbruch(k)) < 1e-12)
        cbuFlag= true;
%         fprintf('Converged... it:%g\n',k);
%             functional = functional(1:k);
%             abbruch = abbruch(1:k);
%             v_array=v_array(1:k,:);
%             sega_array=sega_array(1:k);
%             constraint_vio=constraint_vio(1:k);
%             break;
    end
    if(cbuFlag)
        outerTermi = norm(U(:)-U_old_outer(:))/norm(U(:));
        if(outerTermi < 1e-12 && constraint_vio(k) < 0.1)
            fprintf('Converged... it:%g\n',k);
            functional = functional(1:k);
            outer = outer(1:k);
            abbruch = abbruch(1:k);
%             v_array=v_array(1:k,:);
            sega_array=sega_array(1:k);
            constraint_vio=constraint_vio(1:k);
            break;
        end
    end
     
end

k


%--------------------------------------------------------------------------
function [ w ] = projection( p )
%--------------------------------------------------------------------------
%PROJECTION computes the point in the
%unit simplex conv(e_1,...,e_n) with minimal distance
%to an arbitrary given point.
%
% p \in R^mn,Q


[mn,Q] = size(p);

%projection of p on hyperplane w_1 + ... + w_n = 1
lambda = (sum(p,2)-1)/Q;
w = p-lambda*ones(1,Q);

subzerodetect = zeros(mn,Q);
subzerodetect(w<0) = 1;

while(sum(subzerodetect(:)) > 0)
    % while projection is not in the simplex:
    % project orthogonally on to the corresponding coordinate hyperplane and
    % and project along the corresponding direction on to the simplex
    direction = zeros(mn,Q);
    direction(w>0) = 1;
    w(w<0)=0;
    
    denominator = sum(direction,2);
    lambda = (sum(w,2)-1) ./ denominator;
    
    w = w - lambda * ones(1,Q) .* direction;
    subzerodetect = zeros(mn,Q);
    subzerodetect(w<0) = 1;
end


%--------------------------------------------------------------------------
function B = coupled_shrinkage(U,D,lambda)
%--------------------------------------------------------------------------
%warning('off','MATLAB:divideByZero');
% In MATLAB is min(1,inf) = 1
B = (1 - min(1,lambda./D)).*U;
%warning('on','MATLAB:divideByZero');

%--------------------------------------------------------------------------
function [H1_l,H1_r] = first_order_differences(m,n)
%--------------------------------------------------------------------------
%derivatives at the right and lower boundary are set to zero (mirroring)

d1 = [-ones(m-1,1); 0];
d2 = ones(m,1);
H1_l = spdiags([d1,d2],[0,1],m,m);

d1 = [-ones(n-1,1); 0];
d2 = ones(n,1);
H1_r = spdiags([d1,d2],[0,1],n,n);



